package company_contract

import (
	"github.com/rdlucklib/rdluck_tools/orm"
	"time"
)

type CompanyContract struct {
	CompanyContractId int       `orm:"column(company_contract_id);pk" description:"客户合同id"`
	ContractType      string    `description:"合同类型：枚举值：'新签合同','续约合同','补充协议'"`
	ProductId         int       `description:"产品id"`
	ProductName       string    `description:"产品名称"`
	CompanyId         int       `description:"客户id"`
	CompanyProductId  int       `description:"客户产品id"`
	ContractCode      string    `description:"合同编码"`
	StartDate         string    `description:"合同开始时间"`
	EndDate           string    `description:"合同结束时间"`
	Money             float64   `description:"合同金额"`
	PayMethod         string    `description:"支付方式"`
	PayChannel        string    `description:"支付渠道"`
	ImgUrl            string    `description:"合同图片，多个用#隔开"`
	CreateTime        time.Time `description:"合同创建时间"`
	ModifyTime        time.Time `description:"合同修改时间"`
	Status            int       `description:"状态"`
	PackageType       int       `description:"套餐类型，0：无，1：大套餐，2：小套餐"`
}

//通过开始日期获取当天开始的合同列表
func GetStartContractListByStartDate(startDate string) (total int64, list []*CompanyContract, err error) {
	o := orm.NewOrm()
	sql := "SELECT * FROM company_contract where start_date = ? AND status = 1 "
	total, err = o.Raw(sql, startDate).QueryRows(&list)
	return
	return
}

type CompanyContractPermission struct {
	Id                int       `orm:"column(id);pk" description:"客户合同id"`
	CompanyId         int       `description:"客户id"`
	CompanyContractId int       `description:"合同id"`
	ChartPermissionId int       `description:"权限id"`
	StartDate         string    `description:"开始日期"`
	EndDate           string    `description:"结束日期"`
	CreateTime        time.Time `description:"创建时间"`
	ModifyTime        time.Time `description:"修改时间"`
}
