package data_manage

import (
	"github.com/rdlucklib/rdluck_tools/orm"
	"time"
)

type EdbDataWind struct {
	EdbDataId  int `orm:"column(edb_data_id);pk"`
	EdbInfoId  int
	EdbCode    string
	DataTime   string
	Value      float64
	Status     int
	CreateTime time.Time
	ModifyTime time.Time
	DataTimestamp int64
}

func AddEdbDataWind(items []*EdbDataWind) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	_, err = o.InsertMulti(1, items)
	return
}

func ModifyEdbDataWind(edbInfoId int64, edbCode, dataTime string, value float64) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` UPDATE edb_data_wind SET edb_info_id=?,value=?,status=1,modify_time=NOW() WHERE edb_code=? AND data_time=? `
	_, err = o.Raw(sql, edbInfoId, value, edbCode, dataTime).Exec()
	return
}

func ModifyEdbDataWindStatus(edbInfoId int64, edbCode string) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` UPDATE edb_data_wind SET edb_info_id=?, status=1,modify_time=NOW() WHERE edb_code=? `
	_, err = o.Raw(sql, edbInfoId, edbCode).Exec()
	return
}

func GetEdbDataWindCount(edbCode, dataTime string) (count int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT COUNT(1) AS count FROM edb_data_wind WHERE edb_code=? AND data_time=? `
	err = o.Raw(sql, edbCode, dataTime).QueryRow(&count)
	return
}

func GetEdbDataWindMaxOrMinDate(edbInfoId int) (min_date, max_date string, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT MIN(data_time) AS min_date,MAX(data_time) AS max_date FROM edb_data_wind WHERE edb_info_id=? `
	err = o.Raw(sql, edbInfoId).QueryRow(&min_date, &max_date)
	return
}
