package data_manage

import (
	"fmt"
	"github.com/shopspring/decimal"
	"hongze/hongze_task/utils"
	"github.com/rdlucklib/rdluck_tools/orm"
	"strconv"
	"strings"
	"time"
)

type EdbInfoCalculateNszydpjjs struct {
	EdbInfoCalculateNszydpjjsId int       `orm:"column(edb_info_calculate_nszydpjjs_id);pk"`
	EdbInfoId                   int       `description:"指标id"`
	EdbCode                     string    `description:"指标编码"`
	FromEdbInfoId               int       `description:"计算指标id"`
	FromEdbCode                 string    `description:"计算指标编码"`
	FromEdbName                 string    `description:"计算指标名称"`
	FromSource                  int       `description:"计算指标来源"`
	FromSourceName              string    `description:"计算指标来源名称"`
	FromTag                     string    `description:"来源指标标签"`
	Sort                        int       `description:"计算指标名称排序"`
	CreateTime                  time.Time `description:"创建时间"`
	ModifyTime                  time.Time `description:"修改时间"`
}

//刷新 N数值移动平均计算
func RefreshCalculateNszydpjjs(edbInfoId, source, formulaInt int, fromEdbInfo *EdbInfo, edbCode string) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	o.Begin()
	defer func() {
		if err != nil {
			o.Rollback()
		} else {
			o.Commit()
		}
	}()

	if err != nil {
		return
	}
	edbInfoIdStr := strconv.Itoa(edbInfoId)

	//计算数据
	var condition string
	var pars []interface{}
	condition += " AND edb_info_id=? "
	pars = append(pars, edbInfoId)

	dataList, err := GetEdbDataListAllV1(condition, pars, source, 0)
	if err != nil {
		return err
	}

	existDataMap := make(map[string]*EdbInfoSearchDataV1)
	for _, v := range dataList {
		existDataMap[v.DataTime] = v
	}

	//计算来源数据

	var fromCondition string
	var fromPars []interface{}
	fromCondition += " AND edb_info_id=? "
	fromPars = append(fromPars, fromEdbInfo.EdbInfoId)

	fromDataList, err := GetEdbDataListAll(fromCondition, fromPars, fromEdbInfo.Source, 0)
	if err != nil {
		return err
	}

	var fromDateArr []string
	fromDataMap := make(map[string]*EdbInfoSearchData)
	for _, v := range fromDataList {
		fromDateArr = append(fromDateArr, v.DataTime)
		fromDataMap[v.DataTime] = v
	}

	addSql := ` INSERT INTO edb_data_calculate_nszydpjjs(edb_info_id,edb_code,data_time,value,create_time,modify_time,status,data_timestamp) values `
	var isAdd bool

	arrLen := len(fromDateArr)
	for ak, av := range fromDateArr {
		//处理第一个值
		var valArr []float64
		if findItem, ok := fromDataMap[av]; ok {
			valArr = append(valArr, findItem.Value)
		} else {
			continue
		}
		if ak+1 != arrLen {
			//处理除第一个值之外的N-1个值
			for i := 1; i < formulaInt; i++ {
				arrIndex := ak + i
				if arrIndex >= arrLen {
					break
				}
				arrVal := fromDateArr[arrIndex]
				if findItem, ok := fromDataMap[arrVal]; ok {
					valArr = append(valArr, findItem.Value)
				} else {
					continue
				}
			}
		}
		valArrLen := len(valArr)
		totalVal := decimal.NewFromFloat(0.00)
		for _, v := range valArr {
			//totalVal = totalVal + v
			newDecimal := decimal.NewFromFloat(v)
			totalVal = totalVal.Add(newDecimal)
		}
		//af := decimal.NewFromFloat(totalVal)
		af := totalVal

		bf := decimal.NewFromFloat(float64(valArrLen))
		val, _ := af.Div(bf).Float64()
		currentDate, err := time.Parse(utils.FormatDate, av)
		if err != nil {
			return err
		}

		timestamp := currentDate.UnixNano() / 1e6
		timestampStr := fmt.Sprintf("%d", timestamp)
		valStr := utils.SubFloatToString(val, 4)

		if existItem, existOk := existDataMap[av]; !existOk {
			addSql += GetAddSql(edbInfoIdStr, edbCode, av, timestampStr, valStr)
			isAdd = true
		} else {
			if existItem != nil && existItem.Value != valStr {
				err = ModifyEdbDataCalculateNszydpjjs(int64(edbInfoId), av, valStr)
				if err != nil {
					return err
				}
			}
		}
	}

	if isAdd {
		addSql = strings.TrimRight(addSql, ",")
		_, err = o.Raw(addSql).Exec()
		if err != nil {
			return err
		}
	}
	return
}

type EdbInfoCalculateNszydpjjsDetail struct {
	EdbInfoCalculateNszydpjjsId int       `orm:"column(edb_info_calculate_nszydpjjs_id);pk"`
	EdbInfoId                   int       `description:"指标id"`
	EdbCode                     string    `description:"指标编码"`
	FromEdbInfoId               int       `description:"计算指标id"`
	FromEdbCode                 string    `description:"计算指标编码"`
	FromEdbName                 string    `description:"计算指标名称"`
	FromSource                  int       `description:"计算指标来源"`
	FromSourceName              string    `description:"计算指标来源名称"`
	FromTag                     string    `description:"来源指标标签"`
	Sort                        int       `description:"计算指标名称排序"`
	CreateTime                  time.Time `description:"创建时间"`
	ModifyTime                  time.Time `description:"修改时间"`
	StartDate                   string    `description:"开始日期"`
	EndDate                     string    `description:"结束日期"`
}

func GetEdbInfoCalculateNszydpjjsDetail(edbInfoId int) (item *EdbInfoCalculateNszydpjjsDetail, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT a.*,b.start_date,b.end_date FROM edb_info_calculate_mapping AS a
			INNER JOIN edb_info AS b ON a.edb_info_id=b.edb_info_id
			WHERE a.edb_info_id=? `
	err = o.Raw(sql, edbInfoId).QueryRow(&item)
	return
}

func GetEdbDataCalculateNszydpjjsByCodeAndDate(edbCode string, startDate string) (count int, err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` SELECT COUNT(1) AS count FROM edb_data_calculate_nszydpjjs WHERE edb_code=? AND data_time=? `
	err = o.Raw(sql, edbCode, startDate).QueryRow(&count)
	return
}

func ModifyEdbDataCalculateNszydpjjs(edbInfoId int64, dataTime, value string) (err error) {
	o := orm.NewOrm()
	o.Using("data")
	sql := ` UPDATE edb_data_calculate_nszydpjjs SET value=?,modify_time=NOW() WHERE edb_info_id=? AND data_time=? `
	_, err = o.Raw(sql, value, edbInfoId, dataTime).Exec()
	return
}
