package yb

import (
	"github.com/rdlucklib/rdluck_tools/orm"
	"time"
)

// ActivityType 活动分类表
type ActivityType struct {
	ActivityTypeId   int       `orm:"column(activity_type_id);pk" description:"活动类型ID"`
	ActivityTypeName string    `description:"活动类型名称"`
	Pid              int       `description:"父级id，默认是：0"`
	Sort             int       `description:"排序字段"`
	FrontIsShow      int8      `description:"是否在前台展示，0：不展示，1：展示"`
	CreateTime       time.Time `description:"创建时间"`
}

// TableName 表名变更
func (activityType *ActivityType) TableName() string {
	return "yb_activity_type"
}

type ActivityTypeList struct {
	ActivityTypeId   int                 `description:"活动类型ID"`
	ActivityTypeName string              `description:"活动类型名称"`
	Pid              int                 `description:"父级id，默认是：0"`
	Sort             int                 `description:"排序字段"`
	FrontIsShow      int8                `description:"是否在前台展示，0：不展示，1：展示"`
	CreateTime       time.Time           `description:"创建时间"`
	ChildList        []*ActivityTypeList `description:"下级活动类型列表"`
}

// GetActivityTypeListByPid 获取最高级的活动分类
func GetActivityTypeListByPid(pid int) (list []*ActivityType, err error) {
	o := orm.NewOrm()
	sql := `select * from yb_activity_type where pid=? order by sort asc ,activity_type_id asc`
	_, err = o.Raw(sql, pid).QueryRows(&list)
	return
}

// GetActivityTypeByActivityTypeId 根据id获取活动分类详情
func GetActivityTypeByActivityTypeId(pid int) (list []*ActivityTypeList, err error) {
	o := orm.NewOrm()
	sql := `select * from yb_activity_type where pid=? order by sort asc ,activity_type_id asc`
	_, err = o.Raw(sql, pid).QueryRows(&list)
	return
}
